// BedData.cpp
// Member function definitions for class BedData
#include <stdlib.h>
#include <iostream.h>
#include <fstream.h>
#include <assert.h>
#include <stdio.h>
#include <time.h>
#include <dir.h>
#include <graphics.h>
#include "beddata.h"

extern char *MFC[];


BedData::BedData(int LBN)
{
  BedNum = LBN;
  NumPVParams = 0;
  NumPSDParams = 0;
  LastNumPVParams = 0;
  FileWrites = 0;
  StartFlag = 0;
  NPChanged = False;
  FirstP = LastP = ThisP = NULL;
}

void BedData::SetBedNum(int LBN)
{
  if (LBN >= 1 && LBN <= 24)
    BedNum = LBN;
  else BedNum = 1;
}

// Update Data creates a linked list of PV parameters within
// each BedData object
// It is NOT called for PSD params

void BedData::UpdateData(byte *DataPtr, int TVal)
{

  int TempNumPVParams;

  DataPtr++;
  NumPVParams = *DataPtr;
  TempNumPVParams = NumPVParams;

  while (TempNumPVParams != 0) {

    ThisP = new ParamNode;
    assert(ThisP != NULL);

    if (FirstP == NULL) {
      FirstP = LastP = ThisP;
    }
    else { // add to end
      LastP->Next = ThisP;
      LastP = ThisP;
      LastP->Next = NULL;
    }

  // Point to first parameter field (MFC)
  DataPtr++;
  ThisP->MFC = *DataPtr;

  // Weed out Parameters to be excluded from Display
  switch (ThisP->MFC) {
    case 0x0B :		// Pulse
    case 0x19 :		// T1
    case 0x1A :		// T2
    case 0x21 :		// BloodT
      ThisP->Display = False;
      break;
    default   :
      ThisP->Display = True;
      break;
  }

  // Can build in PSLO here if required

  DataPtr += 2;
  ThisP->Val = *DataPtr;

  DataPtr++;
  ThisP->Type = (*DataPtr) & 0x80;

  ThisP->MCC = (*DataPtr) & 0x07;

  DataPtr++;
  // PV1
  ThisP->Value1 = (*DataPtr)*256;
  DataPtr++;
  ThisP->Value1 += (*DataPtr);
  // correct the sign, if required
  if ((ThisP->Value1 > 511) && ((ThisP->MCC) & 0x01 == 1))
    ThisP->Value1 -= 1024;


  if (ThisP->Type == 0x80) { // triple valued
    DataPtr++;
    ThisP->Value2 = (*DataPtr)*256;
    DataPtr++;
    ThisP->Value2 += (*DataPtr);
    if ((ThisP->Value2 > 511) && ((ThisP->MCC) & 0x01 == 1))
      ThisP->Value2 -= 1024;
    DataPtr++;
    ThisP->Value3 = (*DataPtr)*256;
    DataPtr++;
    ThisP->Value3 += (*DataPtr);
    if ((ThisP->Value3 > 511) && ((ThisP->MCC) & 0x01 == 1))
      ThisP->Value3 -= 1024;

  }

  TempNumPVParams--;
 } // while NumParams != 0
 TempVal = TVal;

 // Detect changes in the number of parameters reported by the bed
 // Ignore the change from initial state (NP = 0) using StartFlag
 if ((NumPVParams != LastNumPVParams) && (StartFlag != 0))
   NPChanged = True;
 else
   NPChanged  = False;

   LastNumPVParams = NumPVParams;

} // UpdateData

// FormatForDisplay formats the parameter value
// Values are either
//  Single or Triple
//  Signed or Unsigned
//  Scaled
//  Valid, Questionable or Useless

void BedData::FormatForDisplay(void)
{
  boolean Scale = True;

  ThisP = FirstP;

  while (ThisP != NULL) {

    // Screen out invalid parameters

    switch (ThisP->Val) {

      case 0x00 : Scale = True;
      		  break;

      case 0x02 : if (ThisP->Type == 0x00) // single
      		    sprintf(ThisP->DisplayValue,"??");
                  else
                    sprintf(ThisP->DisplayValue,"???/??(??)");

      		  Scale = False;	// no scaling required
                  break;

      case 0x03 : if (ThisP->Type == 0x00)
      		    sprintf(ThisP->DisplayValue,"--");
                  else
                    sprintf(ThisP->DisplayValue,"---/--(--)");

      		  Scale = False;
                  break;

      default : break;

    } // switch


    if (Scale) {

    switch (ThisP->MCC) {

      case 0x00 :	// unsigned 0 to 1023
      case 0x01 :	// signed   -512 to +511
        if (ThisP->Type == 0x00)
          sprintf(ThisP->DisplayValue,"%d",ThisP->Value1);
        else sprintf(ThisP->DisplayValue,"%d/%d(%d)",
        	ThisP->Value1,ThisP->Value2, ThisP->Value3);
        break;

      case 0x02 :	// unsigned 0 to 102.3
      case 0x03 :	// signed -51.2 to +51.1
        // single valued only, triple are all integer values
        // except CO which is dealt with later
        sprintf(ThisP->DisplayValue,"%d.%d",
	ThisP->Value1/10, ThisP->Value1%10);
        break;

      case 0x04 :       // unsigned 0 to 10.23
      case 0x05 :	// signed -5.12 to +5.11
        sprintf(ThisP->DisplayValue,"%d.%d%d",
        ThisP->Value1/100, ThisP->Value1%100, ThisP->Value1%10);
        break;
    }

    } // endif Scale

    // Deal with Stored Params, CO and PAWP

    switch (ThisP->MFC) {

      case 0x1F :	// CO
      case 0x20 :       // PAWP
        // check validity
        if (ThisP->Val == 3) {
          sprintf(ThisP->DisplayValue,"--");
          sprintf(ThisP->DisplayTime," at --:--");
          break;
        }
        else if (ThisP->Val == 2) {
          sprintf(ThisP->DisplayValue,"??");
          sprintf(ThisP->DisplayTime," at ??:??");
          break;
        }

        // apply scaling

        if (ThisP->MCC == 0x02 || ThisP->MCC == 0x03)
          sprintf(ThisP->DisplayValue,"%d.%d",
	  ThisP->Value1/10, ThisP->Value1%10);

        else if (ThisP->MCC == 0x04 || ThisP->MCC == 0x05)
          sprintf(ThisP->DisplayValue,"%d.%d",
          ThisP->Value1/100, ThisP->Value1%100);

        else sprintf(ThisP->DisplayValue,"%d",ThisP->Value1);

        // add leading 0 to minutes
        ThisP->Value3 >= 10 ?
          sprintf(ThisP->DisplayTime," at %d:%d",
	  ThisP->Value2, ThisP->Value3) :
          sprintf(ThisP->DisplayTime," at %d:0%d",
	  ThisP->Value2, ThisP->Value3);

        break;

      default :	break;

     } // switch

     ThisP = ThisP->Next;

  } // while ThisP != NULL


} // FormatForDisplay

// Display Data steps through the Parameter list, displays each
// param and then deletes the list

void BedData::DisplayData(void)
{
  setviewport(22,72,278,458,1);
  clearviewport();

  char TextStr[30];
  int x,y = 42;

  sprintf(TextStr,"Bed");
  settextjustify(LEFT_TEXT,CENTER_TEXT);
  settextstyle(DEFAULT_FONT,HORIZ_DIR,1);
  setcolor(LIGHTCYAN);
  outtextxy(100,20,TextStr);
  sprintf(TextStr,"%d",BedNum);
  settextstyle(DEFAULT_FONT,HORIZ_DIR,2);
  outtextxy(132,20,TextStr);

  while (FirstP != NULL) {

    if (FirstP->Display == True) {

      // display Parameter name
      SetTextColor(FirstP->MFC);
      settextjustify(LEFT_TEXT,CENTER_TEXT);
      settextstyle(DEFAULT_FONT,HORIZ_DIR,1);
      sprintf(TextStr,"%s ",MFC[FirstP->MFC]);
      x = 8;
      outtextxy(x,y,TextStr);

      // display value
      x += 48;
      settextstyle(DEFAULT_FONT,HORIZ_DIR,2);

      // catch stored params with time: CO
      if (FirstP->MFC == 0x1F) {
        outtextxy(x,y,FirstP->DisplayValue);
        settextstyle(DEFAULT_FONT,HORIZ_DIR,1);
        outtextxy(x+72,y,FirstP->DisplayTime);
      }
      else if (FirstP->MFC == 0x20) { // PAWP
        outtextxy(x,y,FirstP->DisplayValue);
        settextstyle(DEFAULT_FONT,HORIZ_DIR,1);
        outtextxy(x+40,y,FirstP->DisplayTime);
      }
      else {
        outtextxy(x,y,FirstP->DisplayValue);
      }
      y +=24;

    } // end if Display = True

    ThisP = FirstP;
    FirstP = FirstP->Next;
    delete ThisP;
  }
  if (TempVal) {
    settextstyle(DEFAULT_FONT,HORIZ_DIR,1);
    setcolor(CYAN);
    sprintf(TextStr,"Temp");
    outtextxy(8,y,TextStr);
    sprintf(TextStr,"%d.%d",TempVal/10,TempVal%10);
    settextstyle(DEFAULT_FONT,HORIZ_DIR,2);
    outtextxy(48,y,TextStr);
  }

} // DisplayData


void BedData::SetTextColor(byte MFC)
{
  switch (MFC) {

      case 0x0A:	setcolor(LIGHTGREEN);	// HR
      			break;
      case 0x1C:
      case 0x1D:
      case 0x1E:    	setcolor(LIGHTGREEN);	// CO2
      			break;

      case 0x11:	setcolor(LIGHTBLUE);	// CVP
      			break;
      case 0x0B:				// Pulse
      case 0x19:				// T1
      case 0x1A:				// T2
       	  		setcolor(LIGHTCYAN);
       			break;
      case 0x27:
      case 0x2F:	setcolor(WHITE);	// SpO2
      			break;
      case 0x14:
      case 0x3F:	setcolor(LIGHTMAGENTA);	// ICP,CPP
       			break;
      case 0x10:	setcolor(LIGHTRED);	// ABP
      			break;

      case 0x13:	setcolor(YELLOW);	// PAP
       			break;
      case 0x16:	setcolor(WHITE);	// NIBP
                        break;
      case 0x1F:
      case 0x20:
      case 0x37:	setcolor(BROWN);        // PAWP,CO,CCO
                	break;


      default:		setcolor(WHITE);	// other
                  	break;
    }

} // SetTextColor


void BedData::WriteHeader(char* PName, char* Stud, char* Diag)
{
  char c;
  time_t t;
  char Heading1[40] = "SDN Data Acquisition Version 1.2";
  char Heading2[20] = "G.Forrest 1998";
  char Heading3[20] = "Patient Name:";
  chdir("C:\\bcdos\\mscproj\\v1_05\\data");
  ofstream outFile ("testdata.csv", ios::out);
  if (!outFile) {
     cerr << "File could not be opened" << endl;
     exit(1);
   }
  // output headings
  outFile <<  Heading1 << endl << Heading2 << endl << Heading3 << ",,";
  outFile << PName << endl << "Study:" << ",," << Stud << endl <<
  	  "Diagnosis:" << ",," << Diag << endl;
  t = time(&t);
  outFile << ctime(&t) << endl;

}

void BedData::WriteParamList(void)
{
  chdir("c:\\bcdos\\mscproj\\v1_05\\data");
  ofstream outFile ("testdata.csv", ios::app);
  outFile << "Time" << ",";
  ThisP = FirstP;
  while (ThisP != NULL) {
    if (ThisP->Type == 0)  outFile << MFC[ThisP->MFC] << ",";
    else outFile << MFC[ThisP->MFC] << ",,,";
  ThisP = ThisP->Next;
  }
  outFile << endl;
  outFile.close();
  chdir("C:\\bcdos\\mscproj\\v1_05");

}  // WriteParamList

void BedData::WriteNewParamList(int NumPVP)
{
 chdir("C:\\bcdos\\mscproj\\v1_05\\data");
  ofstream outFile("testdata.csv",ios::app);
  if (!outFile) {
    cerr << "File could not be opened" << endl;
    exit(1);
  }
  outFile << "Parameter list changed. There is/are now " << NumPVP <<
            " parameter(s)" << endl;
  outFile << "Time" << ",";
  ThisP = FirstP;
  while (ThisP != NULL) {
    if (ThisP->Type == 0x00)
       outFile << MFC[ThisP->MFC] << ",";
    else
       outFile << MFC[ThisP->MFC] << ",,,";
    ThisP = ThisP->Next;
  }
  outFile << endl;
  outFile.close();
  chdir("c:\\bcdos\\mscproj\\v1_05");

} // WriteChangeMessage


void BedData::WriteParamsToFile(void)
{

  struct tm *tblock;
  time_t timer;
  chdir("C:\\bcdos\\mscproj\\v1_05\\data");
  ofstream outFile ("testdata.csv", ios::app);
  if (!outFile) {
     cerr << "File could not be opened" << endl;
     exit(1);
  }
  timer = time(NULL);
  tblock = localtime(&timer);
  outFile << tblock->tm_hour << ":";
  (tblock->tm_min < 10) ?
     outFile << "0" << tblock->tm_min << "," :
     outFile << tblock->tm_min << ",";

  ThisP = FirstP;
  while (ThisP != NULL) {
    if (ThisP->Type == 0x00)
       outFile << ThisP->Value1 << ",";
    else
      outFile << ThisP->Value1 << "," << ThisP->Value2 << ","
              << ThisP->Value3 << ",";
    ThisP = ThisP->Next;
  } // while ThisP != NULL)
  // delete the parameter list
  while (FirstP != NULL) {
    ThisP = FirstP;
    FirstP = FirstP->Next;
    delete ThisP;
  }
  outFile << endl;
  outFile.close();
  chdir("C:\\bcdos\\mscproj\\v1_05");

}

// Utilities
int BedData::CheckStartFlag(void)
{
  if (StartFlag == 0) return 0;
  else return 1;
}

void BedData::SetStartFlag(void) {StartFlag = 1;}
void BedData::ClearStartFlag(void) {StartFlag = 0;}

int BedData::CheckNPChanged(void)
{
  if (NPChanged) return 1;
  else return 0;
}

int BedData::GetNumPVParams(void) { return NumPVParams;}

