// CPLDRVFN.CPP

#include <time.h>
#include <dos.h>
#include <conio.h>

#ifndef GENERAL_H
 #include "general.h"
#endif

#define XMIT_SIZE 500   // size of transmit
#define RECV_SIZE 3000  // and receive buffers

// ===== defined in ASSDRV_B.ASM ========

extern "C" {
  unsigned CplInit (unsigned, unsigned);
  void CplWrite (byte *, unsigned);
  unsigned CplRead (byte *, unsigned);
}

int TimedCplRead(byte [], int);
void ErrExit(byte[],int);

// --------------------------------------------------------
//	StartCP() Allocates shared memory from DOS, then
//	resets and tests the interface board.
//	Returns 1 if successful, 0 otherwise.
// --------------------------------------------------------

short StartCP(void)
{

  long StartOnlineTimeout,EndOnlineTimeout;

  byte Buffer[26];
  word ShMemSeg;		// segment address returned by
                        // CplInit

  // Data Link Layer Requests

  byte UnitStatus[] = {0xAF,0,0,0};	// reply is BFh, 26 bytes

  byte ResetAndTest[] = {0xA0,0,0,0};	// reply is B0h, 4 bytes

  textcolor(LIGHTCYAN);
  gotoxy(1,15);
  cprintf("CAREPLANE SDN DRIVER V1.05\n\r");
  cprintf("G.Forrest 6/10/98\n\r\n\r");

  // ----------------------------
  // 	Allocate shared Memory
  // ----------------------------

  textcolor(WHITE);
  cprintf("Setting up Shared Memory...\n\r\n\r");

  ShMemSeg = CplInit(XMIT_SIZE, RECV_SIZE);

  textcolor(LIGHTGRAY);

  if (ShMemSeg == 0) {
    cprintf("Careplane refused to allocate shared memory.\n\r");
    return(0);
  } // endif
  else {
    cprintf("DOS has allocated shared memory.\n\r");
    cprintf("The shared segment address is: %dH\n\r\n\r",ShMemSeg);
  }

  // ---------------------------------------------
  // 	Send a UNIT STATUS message to the board
  // ---------------------------------------------

  textcolor(WHITE);
  cprintf("Sending the first UNIT STATUS message to Careplane...\n\r\n\r");

  textcolor(LIGHTGRAY);

  CplWrite(UnitStatus,2);

  if (!TimedCplRead(Buffer,2)) {

    CplWrite(ResetAndTest,2);
    if (!TimedCplRead(Buffer,2)) return 0;
    cprintf("Reset command sent. You should now restart.\n\r");
    return 0;
  }

  if (Buffer[0] != 0xBF) {
    cprintf("Invalid response\n\r");
    ErrExit(Buffer,4);
    return 0;
  }

  else if ((Buffer[2] & 0xF0) != 0x10) {
    cprintf("Request denied\n\r");
    ErrExit(Buffer,4);
    return(0);
  } // endif

  else if (!TimedCplRead(&Buffer[4],11)) return 0;

  else  cprintf("Request accepted\n\r\n\r");

  // ------------------------------------------
  //	Send RESET and TEST message to board
  // ------------------------------------------

  textcolor(WHITE);
  cprintf("Now sending a RESET AND TEST message to Careplane...\n\r\n\r");

  textcolor(LIGHTGRAY);

  CplWrite(ResetAndTest,2);

  if (!TimedCplRead(Buffer,2)) return 0;

  if (Buffer[0] != 0xB0) {
     cprintf("Invalid response\n\r");
     ErrExit(Buffer,4);
     return 0;
  }
  else if ((Buffer[2] & 0xF0) != 0x10) {
    cprintf("Request denied\n\r");
    ErrExit(Buffer,4);
    return 0;
  }
  else cprintf("Request accepted\n\r\n\r");

  // -------------------------------------------------------
  //	Send more UNIT STATUS messages until board clears
  //	RSI (Restart Indicator) which takes ~ 6 seconds
  // -------------------------------------------------------

  textcolor(WHITE);

  cprintf("Now sending more UNIT STATUS messages...\n\r\n\r");

  textcolor(LIGHTGRAY);

  do {

    time(&StartOnlineTimeout);

    CplWrite(UnitStatus,2);

    if (!TimedCplRead(Buffer,13)) return 0;

    if ( (Buffer[0] == 0xBF) && (Buffer[5] == 0x1C) ) {
      textcolor(LIGHTGREEN);
      cprintf("Careplane board initialised\n\r");
      sleep(2);
      return 1;
    } //endif

    time(&EndOnlineTimeout);

  } while (EndOnlineTimeout - StartOnlineTimeout < TIMEOUT);

  cprintf("Failed get Careplane Board On Line after Reset\n\r");
  ErrExit(Buffer,9);
  return(0);


} // StartCP

// -------------------------------------------------------
// TimedCplRead() is a variation on CplRead that times out
// and returns 0 if nothing read after 12 seconds
// arguments:   byte Buffer[] address for data
//              int NumWords - the number of words to read
// -------------------------------------------------------

int TimedCplRead(byte Buffer[],int NumWords)
{
  long StartTimeout = 0,EndTimeout = 0;

  time(&StartTimeout);

  while (CplRead(Buffer,NumWords) == 0) {

   time(&EndTimeout);

   if (EndTimeout - StartTimeout > TIMEOUT) {
     cprintf("Buffer read timed out after %d seconds\n\r",TIMEOUT);
     return 0;
   } // endif

  } // while
  return 1;
}

// --------------------------------------------
// ErrExit() displays the invalid Buffer values
// --------------------------------------------

void ErrExit(byte Buffer[], int max)
{
  int i;
  for (i = 0; i < max; i++)
    cprintf("Buffer[%d] = %d\n\r",i,(int)Buffer[i]);
}