#ifndef DATAMSG_H
#define DATAMSG_H

#ifndef GENERAL_H
  #include "general.h"
#endif

// -------------------------------------------
// 	Logical Source Data Message
// 	  Consists of Preamble	(bytes 0-3)
//	  Element Header	(bytes 4-9)
//	  SDN message, the actual data
// -------------------------------------------

struct Preamble {
  byte	MsgID,		// 0x1F for logical source data messages
        LSN,		// logical source identifier 0-F only
        Flags,		// b3=wave, b2=overrun, b0=full
        AZero,
        Length;		// if WVF=0,length of current element in bytes
        		// actually part of element header, but including
                        // here can detect NULL messages.
};

struct ElementHeader {
  byte	Length,		// as above, if WVF=1,no. of wave packets
  	Timing,		// b7-5 Time Ref, b4-0 Logical poll slot no.
	LBN,		// logical bed no.
	SDNMsg:4,	// message class 0-A in b3-0
        PS:2,		// phsiological source (mon/comp) b5-4
        ZeroA:1,
        TST:1,		// TST=1 if in test mode
        MFC:6,		// Medical function code b5-0
        ZeroB:1,
        TODS:1,		// Time of day source
        PSLO;		// physiological source local offset 0-F
        		// module slot within monitor
};

// ------------
// SDN messages
// ------------

// -------------------------
//	Instrument Status
// -------------------------

struct ISDMsg { 	// instrument status data message
  word	Res1;		// reserved
  byte	Res2,		// reserved
  	HR_TEL,         // b2=heart rate, b1-0=telemetry enable
        Res3,		// reserved
        Flags,		// b5=STS,b4=CS,b3=DLS,b2=SCS,b1=SI,b0=RSI
        AR_CG,		// b3=alarm reception,b2-0=care group assignment
        ZeroA;
  word  Res4,
  	Res5;
  byte  NP:4,		// b3-0 no.of parameters(0-F)
  	NW:4,		// b7-4 no.of waves
  	MaxPSLO;	// no.of local PS's allocated to this unit
};

struct ISData {         // instrument status data - indicates the PV
			// and wave data that are available
  byte	WPFC,		// wave/parameter function code-an MFC that
  			//  identifies a wave or parameter (0 - 0x3F)
  	PSLO;		// module generating this data (0-15)
};


// -----------------------
// Parameter Value Message
// -----------------------


struct PVData {

  byte  MFC,		// Parameter MFC
        PSLO,
	Val,		// Data validity
        MCC:3,		// Magnitude conversion code
        CC:1,		// PSD change counter
        LIM:1,		// Alarm limit conditions
        ALM:1,		// Alarm conditions
        PA:1,		// Parameter alarms
        TYP:1,		// 0=single, 1=triple
        PV1h,		// High order 2 bits of PV1
        PV1l,		// lower 8 bits of PV1
        		// For single valued params, message ends
	PV2h,		// High order 2 bits of PV2
        PV2l,		// lower byte of PV2
        PV3h,		// High order 2 bits of PV3
        PV3l;		// lower byte of PV3
};

struct PSData {         // Parameter Support Data
  byte 	Val,    	// validity
        LL:3,		// Label Length
        PA:1,		// Parameter alarms
        TYP:2,		// 0:RT numeric, 1:text only, 2:Stored numeric
        LIM:1,		// Alarm limit condition
        ALM:1,		// O:No alarm, 1:alarm condition
        Maxh,		// max range
        Maxl,
        Minh,
        Minl;
  char  Text[21];

};



struct BedNode {

  byte BedNum;
  BedNode *NextBed;

};


#endif