// GetBeds.CPP
// Display a list of active beds


#include <conio.h>
#include <assert.h>
#include <stdlib.h>
#include <iostream.h>
#include "getbeds.h"

#ifndef GENERAL_H
  #include "general.h"
#endif

#ifndef DATAMSG_H
  #include "datamsg.h"
#endif


int GetActiveBeds(void)
{
  byte Buffer[200];	// temp storage for Data Messages
  ElementHeader *ElHead;
  ISDMsg *ISD;
  byte *DataPtr;
  BedNode *FirstBed, *LastBed, *ThisBed;
  FirstBed = LastBed = ThisBed = NULL;
  boolean ListComplete = False;
  boolean BedFound = False;
  int BedToRecord;

  textmode(C4350);
  _setcursortype(_NOCURSOR);
  clrscr();
  textcolor(WHITE);
  cprintf("The following Beds are Active:\n\r\n\r");

  // PollLS has read 2 words (preamble) and confirmed a Data Message

  // Read the Length of the 1st element

  if (!TimedCplRead(Buffer,1)) return 0;

  // While length > 6, read the rest of the header, then the message

  while (Buffer[0] > 6) {  // 6 is the min length that can be read

    if (!TimedCplRead(&Buffer[2],2)) return 0;

    ElHead = (ElementHeader*)Buffer;

    // read the rest of the message

    if (!TimedCplRead(&Buffer[6],(ElHead->Length/2)-3)) return 0;

    // check the Message Class

    if (ElHead->SDNMsg != 0) {
      cprintf("Not an Intrument Status Message!");
      return 0;
    }

    ISD = (ISDMsg*)&Buffer[6];

    // check the Status Flags

    if (ISD->Flags != 0x1C) {
      cprintf("Bedside Monitor not ready.");
      return 0;
    }

    DataPtr = (byte*)ISD + (sizeof(ISDMsg));

    if (ListComplete == False) {

      // create a new BedNode

      ThisBed = new BedNode;
      assert(ThisBed != NULL);
      ThisBed->BedNum = ElHead->LBN;

      switch (FirstBed == NULL) {

        case True : FirstBed = LastBed = ThisBed;
                    DisplayBedStatus(DataPtr,ISD->NW,ISD->NP,ElHead->LBN);
      		    break;

        case False : // Check for the first bed coming around again
    		     // and skip display/delete the bed

                     if (FirstBed->BedNum == ThisBed->BedNum) {
       		       ListComplete = True;
                       delete ThisBed;
                       break;
                     }
                     else {
                       DisplayBedStatus(DataPtr,ISD->NW,ISD->NP,ElHead->LBN);
		       LastBed->NextBed = ThisBed;
      		       LastBed = ThisBed;
                       LastBed->NextBed = NULL;
                       break;
		     }
      } // switch

    } // endif ListComplete == False

    // get length of next element

    if (!TimedCplRead(Buffer,1)) return 0;


  } // while Length > 6

  // All elements and messages have now been read


  do {

    BedToRecord = GetBedFromUser();

    if (BedToRecord == 0) {
      // quit from Active Bed Display
      return 25;
    }

    // check that the bed is in the list
    ThisBed = FirstBed;

    do {

      if (ThisBed->BedNum == (byte)BedToRecord) {
        BedFound = True;
        break;
      }
      else
        ThisBed = ThisBed->NextBed;

    } while (ThisBed != NULL);

  } while (BedFound == False);

  // delete the Bed List

  while (FirstBed != NULL) {

    ThisBed = FirstBed;
    FirstBed = FirstBed->NextBed;
    delete ThisBed;

  }

  textmode(C80);
  return BedToRecord;
}

void DisplayBedStatus(byte *WPPtr, byte NumW, byte NumP, byte BedNum)
{
  ISData *ISDat = (ISData*)WPPtr;

  textcolor(YELLOW);
  cprintf("Bed %d\n\r",BedNum);

  textcolor(LIGHTCYAN);
  cprintf("Waves: ");

  while (NumW != 0)  {

    if (ISDat->WPFC != 0)
      cprintf("%s ", MFC[(int)ISDat->WPFC]);
    ISDat++;
    NumW --;
  }
  textcolor(LIGHTGREEN);
  cprintf("\n\rParameters: ");

  while (NumP != 0) {

    if (ISDat->WPFC != 0)
      cprintf("%s ",MFC[(int)ISDat->WPFC]);
    ISDat++;
    NumP--;
  }
  cprintf("\n\n\r\r");
}


int GetBedFromUser(void)
{
  long BedSelected;
  char *cinstr, c;

  gotoxy(61,49);
  cprintf("         ");
  textcolor(WHITE);
  gotoxy(1,49);
  cprintf("Select a bed to record from, or q to quit, then press <return>: ");
  cin.get(cinstr,3,'\n');
  BedSelected = strtol(cinstr,NULL,0);

  // Flush the input stream

  do {
   c = cin.get();
  } while (c != '\n');

  return (int)BedSelected;
}