// Session layer
// Requests to: Connect
//              Activate
//		Suspend
//		Poll
//		Disconnect
// Logical Sources

#include <iostream.h>
#include <time.h>
#include <conio.h>
#include "logsourc.h"

#ifndef GENERAL_H
  #include "general.h"
#endif

#ifndef SRCREQ_H
  #include "srcreq.h"
#endif



// ----defined in ASSDRV_B.ASM-----

extern "C" void CplWrite (byte *, unsigned);

// ----defined in CPLDRVFN.CPP-------

extern int TimedCplRead(byte [], int);
extern void ErrExit(byte[],int);

// -----------------------------------

int i;

// ------------------------------
// 	Connect Logical source
// ------------------------------

int ConnectLS(ConLSReq *ConLSR)
{

 byte Buffer[4];

 CplWrite( (byte*)ConLSR, sizeof(ConLSReq)/2 );

 if (!TimedCplRead(Buffer,2)) return 0;

 if ( (Buffer[0] == 0x10) && ( (Buffer[2] & 0x10) == 0x10) ) return(1);

 else {                 	// connection refused

   cout << "Connect Logical Source Failed" << endl;
   Buffer[2] &= 0x0F;
   switch ((int)Buffer[2]) {

     case 0x00 :	cout << "Syntax error";
     			break;
     case 0x01 :	cout << "Source not connected";
     			break;
     case 0x02 :	cout << "Source not activated";
     			break;
     case 0x04 :	cout << "Source already exists";
     			break;
     case 0x05 :	cout << "Directory full";
     			break;
     case 0x06 :	cout << "Insufficient buffer space";
     			break;
     case 0x07 :	cout << "Insufficient capability";
     			break;
     case 0x0E :	cout << "Illegal use (Careplane is offline)";
     			break;
     case 0x0F :	cout << "Unimplemented specification in request";
     			break;
     default :		cout << "Error descriptor not recognised";
     			break;
   } // switch

   cout << endl;

   // process extended error codes

   switch ((int)Buffer[3]) {

     case 0x01 :	cout << "Zero fields in Request were not zero";
     			break;
     case 0x02 :	cout << "LSN is illegal";
     			break;
     case 0x03 :	cout << "No message class specified";
     			break;
     case 0x04 :	cout << "No beds specified";
     			break;
     case 0x05 :	cout << "COM bit not set for TOD, BL,PNI";
     			break;
     case 0x06 :	cout << "Wave list position must not be 0";
     			break;
     case 0x08 :	cout << "More than 1 bed not allowed for waves";
     			break;
     case 0x09 :	cout << "WV must be the only message class if waves"
     			<< " are requested";
                        break;
     case 0x0A :	cout << "Only one source is allowed for waves";
     			break;
     case 0x0B :	cout << "LS must be in continuous automatic if " <<
     			"waves are requested";
                        break;
     case 0x0C :	cout << "Wave compression only allowed for waves";
     			break;
     default :		cout << "Extended error code not recognised";
     			break;
   } // switch

   cout << endl;
   return 0;
 }

} // Connect LS


// -------------------------------
// 	Activate Logical Source
// -------------------------------

int ActivateLS(int LogSrcNum)
{
  byte Buffer[4];
  ActLSReq ActLS1 = DefActLSReq;
  ActLS1.LSN = (byte)LogSrcNum;

  ActLSReq *ActLS1p = &ActLS1;

  CplWrite( (byte*)ActLS1p, sizeof(ActLSReq)/2 );

  if (!TimedCplRead(Buffer,2)) return 0;

  if ( (Buffer[0] == 0x11) && ((Buffer[2] & 0xF0) == 0x10) ) return(1);

  else cout << "Activate Logical Source Failed" << endl;

  if ((Buffer[0] == 0x11) && ((Buffer[2] & 0xF0) == 0x00)) {
    switch ((int)Buffer[2] & 0x000F) {
      case 0x00 :  	cout << "Syntax error";
                        break;
      case 0x01 :	cout << "Logical source not connected";
      			break;
      case 0x03 :	cout << "Logical source not suspended";
      			break;
      case 0x07 :	cout << "Insufficient capability";
      			break;
      case 0x0E :	cout << "Illegal use (Careplane is offline)";
      			break;
      default   :	cout << "Error code not recognised";
      			break;
    }
  }
  else cout << "Invalid response to request";

  cout << endl;
  ErrExit(Buffer,4);
  return(0);


} // ActivateLS

// ---------------------------------
// 	Poll Logical Source
// ---------------------------------

int PollLS(int LogSrcNum)
{

  byte Buffer[4];

  PollLSReq[1] = (byte)LogSrcNum;

  CplWrite( PollLSReq,2 );

  if (!TimedCplRead(Buffer,2)) return 0;

  if ( Buffer[0] == 0x1F ) return(1);	// data message begins 0x1F

  // a response is sent if an error is detected

  else if (Buffer[0] == 0x13) {
    cout << "Poll Logical Source Request was denied." << endl;
    cout << "Reason: ";
    switch (Buffer[2]) {
     case 0x00 : 	cout << "Syntax error in request\n";
                	break;
     case 0x01 :	cout << "Logical source not connected\n";
     			break;
     case 0x02 :	cout << "Logical source not active\n";
     			break;
     case 0x0E :	cout << "Illegal use, source not in poll mode\n" <<
     			"or careplane offline\n";
                        break;
     default :		cout << "Error Descriptor not recognised\n";
     			break;
    }; // switch

    return(0);
  }

  else {
    cout << "Message didn't begin with 1FH" << endl;
    ErrExit(Buffer,10);
    return(0);
  }

} // PollLS


// ---------------------------------
// 	Suspend Logical Source
// ---------------------------------

int SuspendLS(int LogSrcNum)
{
  byte Buffer[4];
  SuspLSReq SuspLSn = DefSuspLSReq;

  SuspLSn.LSN = (byte)LogSrcNum;

  SuspLSReq *SuspLSnp = &SuspLSn;

  cout << "Sending Suspend command..." << endl;

  CplWrite ( (byte*)SuspLSnp, sizeof(SuspLSReq)/2 );

  if (!TimedCplRead(Buffer,2)) return 0;

  if ( (Buffer[0] == 0x12) && ((Buffer[2] & 0xF0) == 0x10) ) return(1);

  else {
    cout << "Suspend Logical Source Failed." << endl;
    ErrExit(Buffer,4);
   return(0);
  }

} // SuspendLS



// ---------------------------------
// 	Disconnect Logical Source
// ---------------------------------

int DisconnectLS(int LogSrcNum)
{
  byte Buffer[4];
  DiscLSReq[1] = (byte)LogSrcNum;	// LSN

  CplWrite( DiscLSReq, 2);

  if (!TimedCplRead(Buffer,2)) return 0;

  if ((Buffer[0] == 0x15) && ((Buffer[2] & 0xF0) == 0x10) ) return(1);  // success

  else {
    cprintf("Disconnect failed.\n\r");
    ErrExit(Buffer,4);
    return(0);
  }

} // DisconnectLS
