// Main.cpp
// Main program
// v1.05

#include <dos.h>
#include <conio.h>
#include <assert.h>
#include <stdlib.h>
#include <graphics.h>
#include <iostream.h>
#include "main.h"
#include "logsource.h"
#include "readmsg.h"
#include "getbeds.h"
#include "beddata.h"

// defined in ASSDRV_B.ASM
extern "C" void CplExit(void);
// defined in SRCREQ.H
extern ConLSReq DefConLSReq;



int main(void)
{

  ConLSReq ISDReq = DefConLSReq;
  ISDReq.LSN = 0x00;
  ISDReq.LB8to1 = 0xFF;			// All Beds
  ISDReq.LB16to9 = 0xFF;
  ISDReq.LB24to17 = 0xFF;
  ISDReq.DataClass1 = 0x01;		// IS Data
  ISDReq.Mode = 0x00;			// Poll

  ConLSReq PVReq = DefConLSReq;
  PVReq.LSN = 0x01;
  PVReq.DataClass1 = 0x0C;		// PV and PS Data
  PVReq.Mode = 0xFF;			// Automatic

  BedData *ThisBed;
  int BedSelected;
  boolean Repeat = False;
  boolean AcqData;
  int Response, Retry = 0;

  char PatientName[25];
  char Study[20];
  char Diagnosis[20];

  textmode(C80);
  clrscr();
  _setcursortype(_NOCURSOR);

  if (!StartCP()) {
    cprintf("Failed to initialise Careplane");
    CplExit();
    return 0;
  }

  do {

  AcqData = True;

  if (!ConnectLS(&ISDReq)) {
    cprintf("Failed to Connect for IS Data");
    CplExit();
    return 0;
  }

  clrscr();
  textcolor(WHITE);
  cprintf("Acquiring data for the Bed List...");

  sleep(2);		// allow time for CPlane to acquire data

  if (!PollLS(0)) {
    cprintf("Failed to Poll LS");
  }

  BedSelected = GetActiveBeds();

  if (!BedSelected) cprintf("Failed to Get Active Beds");

  textmode(C80);
  _setcursortype(_NOCURSOR);
  clrscr();
  cprintf("Bed Selected = %d\n\r\n\r",BedSelected);

  if (!DisconnectLS(0)) {
    cprintf("Failed to Disconnect LS");
    CplExit();
    return 0;
  }

  if ((BedSelected >= 1) && (BedSelected <= 8)) {
    PVReq.LB8to1 = (byte)ConvertBedToHex(BedSelected);
    PVReq.LB16to9 = 0x00;
    PVReq.LB24to17 = 0x00;
  }

  else if ((BedSelected >= 9) && (BedSelected <= 16)) {
    PVReq.LB16to9 = (byte)ConvertBedToHex(BedSelected - 8);
    PVReq.LB8to1 = 0x00;
    PVReq.LB24to17 = 0x00;
  }
  else if (BedSelected >= 17 && BedSelected <= 24) {
    PVReq.LB24to17 = (byte)ConvertBedToHex(BedSelected - 16);
    PVReq.LB8to1 = 0x00;
    PVReq.LB16to9 = 0x00;
  }
  else if (BedSelected == 25)  // no parameter acquisistion
    AcqData = False;


  if (AcqData) {


    ThisBed = new BedData;
    assert (ThisBed != NULL);

    textcolor(LIGHTGRAY);
    cprintf("Patient Name: ");
    cin.get(PatientName,25,'\n');
    FlushCin();
    cprintf("Study: ");
    cin.get(Study,20,'\n');
    FlushCin();
    cprintf("Diagnosis: ");
    cin.get(Diagnosis,20,'\n');
    FlushCin();

    ThisBed->WriteHeader(PatientName, Study, Diagnosis);

    textcolor(WHITE);
    cprintf("Acquiring data for Bed %d...",BedSelected);
    sleep(2);

    if (!ConnectLS(&PVReq))
      cprintf("Failed to Connect for Parameter Values");

    ThisBed->SetBedNum(BedSelected);

    if (!DrawScreen()) return 0;

    switch (ReadMsg(ThisBed)) {

      case 1 : 	closegraph();
                textmode(C80);
                break;
      case 0 :  closegraph();
      		textmode(C80);
                cprintf("Buffer Read Failure\n\r");
                Exit();
                return 0;
      case 2 :  closegraph();
      		textmode(C80);
                cprintf("Not a PV/PSD message, source in self test"
                " or not a monitor\n\r");
                Exit();
                return 0;
      case 3 :  closegraph();
      		textmode(C80);
                cprintf("Disconnect failed\n\r");
                Exit();
                return 0;
      case 4 :	closegraph();
      		textmode(C80);
                cprintf("Another message type (Not Data or DISC_RSP)\n\r");
                Exit();
                return 0;
      default : closegraph();
      		textmode(C80);
                cprintf("Read message return value not in range 0 - 4\n\r");
                Exit();
                return 0;
      } // switch

  } // endif AcqData

  do {

    _setcursortype(_NOCURSOR);
    textbackground(BLACK);
    textcolor(LIGHTMAGENTA);
    clrscr();
    gotoxy(16,12);
    while (kbhit()) getch();
    cprintf ("Would you like to view the bed list again? Y/N ");
    Response = getch();

    switch (Response) {

      case 'Y': case 'y':	Repeat = True;
      				Retry = 0;
				break;
      case 'N': case 'n':	Repeat = False;
      				Retry = 0;
                        	break;

      default :			Retry = 1;
      				break;
    } // switch


  } while (Retry);

 delete ThisBed;

 } while (Repeat);


  CplExit();
  textcolor(WHITE);
  _setcursortype(_NORMALCURSOR);
  clrscr();
  cprintf("Program terminated normally");
  textcolor(LIGHTGRAY);
  return 1;
}


// ConvertBedToHex takes a Bed Number and returns the appropriate
// value in hex for the ConnectLS request

int ConvertBedToHex(int BedSel)
{
  int i,pow2 = 1;

  for (i = 1; i <= (BedSel - 1); i++ ) {

    pow2 *= 2;
  }

  return pow2;
}

void FlushCin(void)
{
  char c;
  do {
    c = cin.get();
  } while (c != '\n');
}

void Exit(void)
{
  CplExit();
  cprintf("Abnormal termination");
}