// Logical Source Command Messages

#ifndef SRCTYPES_H
#define SRCTYPES_H

#ifndef GENERAL_H
  #include "GENERAL.H"
#endif

// ----------------
// Request ID codes
// ----------------

const CONN_REQ_ID = 0x00;		// connect
const DISC_REQ_ID = 0x05;               // disconnect
const ACT_REQ_ID  = 0x01;               // activate
const SUSP_REQ_ID = 0x02;		// suspend
const POLL_REQ_ID = 0x03;		// poll
const SCHD_REQ_ID = 0x04;		// schedule
const DIRS_REQ_ID = 0x07;		// directory status

// -----------------
// Response ID codes
// -----------------

const CONN_RESP = 0x01;		// connect response
const DATA_MSG  = 0x1F;		// data message
const UNST_RESP	= 0xBF;		// unit status response
const DISC_RESP = 0x15;		// disconnect response

// -------------------
// SDN Message Classes
// -------------------

const INST_STAT = 0x00;
const PAR_VAL = 0x02;


// -------------------------------
// Logical source command messages
// -------------------------------


// Connect Logical Source Request

struct ConLSReq {
  byte	MsgID,		// id = 00
  	LSN,		// 3-0 logical source number
  	Mode,	        // 0=Poll, 1-0xFE=Sched.Automatic,
  			// 0xFF=Continuous automatic
	AuxMode,	// b7=IAF (1=inh),b6=AModeSel,b5-0=Sched Period
	Contype,	// must be 0
        LB8to1,		// logical beds 1-8
        LB16to9,	// logical beds 16-9
        LB24to17,	// logical beds 24-17
        Com_Mon,	// bit2=incl.computer data,bit1=incl.monitor data
        DataClass1,	// bits7-0: WV,WSD,IT,AT,PSD,PV,AS,IS
        DataClass2,	// bits2-0=PNI,BL,TOD
        MFC,		// 0=All codes, 1-0x3F=specific MFC
        PSLO,		// 0=All PSLO, 1-0x3F=specific PSLO
        Misc;		// bit7=ECG CMPR,6=TOD source, 2-0 WLP
};


// Activate Logical Source Request

struct ActLSReq  {
  byte	MsgID,
        LSN,
        ZeroA,
        ZeroB;
};


// Suspend Logical Source Request

struct SuspLSReq  {
  byte	MsgID,
  	LSN,
        ZeroA,
        ZeroB;
};




#endif